#include <stdlib.h>
#include <conio.h>
#include <peekpoke.h>
#include <vic20.h>
#include <string.h>

#include "common.h"
#include "structs.h"
#include "vis-sss-bridge.h"

// Game constants
#define MULTICOLOR 0x08
#define SCREENDATA_ENDMARKER  0x40
#define SCREENDATA_RLEINDICATOR 0x80
#define PLAYFIELD_WIDTH (22 * 8 )
#define PLAYFIELD_HEIGHT (23 * 8) 
#define PLAYFIELD_BOUNDS_LEFT 16 
#define PLAYFIELD_BOUNDS_RIGHT (PLAYFIELD_BOUNDS_LEFT +  PLAYFIELD_WIDTH)
#define PLAYFIELD_TOP 16
#define PLAYFIELD_BOTTOM (PLAYFIELD_TOP + PLAYFIELD_HEIGHT)
#define PLAYAREA_BOTTOM (PLAYFIELD_BOTTOM - 16)
#define BIGDUDE_HEIGHT = 16

#define  KB_PREV_KEY_LOC 203
#define  KB_SHIFT_LOC 145
#define LASER_IDLE 127


// External data 
extern BYTE BIGDUDEBITS[];
extern BYTE SCREENDATA[];
extern BYTE COLORDATA[];
extern BYTE PLAYFIELD[];
extern BYTE PLAYCOLOR[];
extern BYTE SPRITEDEF[];
extern BYTE SPRITEX[];
extern BYTE SPRITEY[];

// Function Prototypes
void plotScreenRLE(BYTE x, BYTE y, BYTE* pCharData, BYTE* pColourData);
void plotValuesRLE(BYTE x, BYTE y, BYTE* pData, BOOL isColour);
void InitScreenGeometry();
void auxColour(BYTE c);
void UpdateMeanies();
void InitGameData();
void InitMeanies();
void ResetMeanie();
void InitGame();
void GameLoop();
void DrawScreen();
void UpdateLasers();

// Static Data
const BYTE  gMovePattern[] = {MAKEBYTE(2,0), MAKEBYTE(2,0), MAKEBYTE(2,0), MAKEBYTE(0,2), MAKEBYTE(0,2), MAKEBYTE(0,2)};
GAMEDATA gd;

void main (void)
{
	BYTE bigDudeSprite = 0;
	BYTE idx = 0;
	srand(0.5);
	
	// Init Vic SSS
	InitScreenGeometry();
	SSSINIT();
	
	// Start IRQ routine for optional frame rate lock	
	IrqSetup();

	// Create meanies
	InitGame();
	
	// Run the game!
	GameLoop();


}
void InitGame()
{
	InitGameData();
	
	DrawScreen();
	
	InitMeanies();
}

void GameLoop()
{
	while(TRUE)
	{
		gd.lastKey = PEEK(203);
		UpdateMeanies(); 
		//UpdateLasers(); 		
		SSSFLIP(1);
	}
}

void InitGameData()
{
	Laser lasers[] = 
					{
						{48,1,3,1,LASER_IDLE},
						{17,1,8,1,LASER_IDLE},
						{33,13,1,LASER_IDLE},
						{49,21, 3,-1,LASER_IDLE},
						{18,21, 8,-1,LASER_IDLE},
						{34,21, 13,-1,LASER_IDLE}
					};
					
	memcpy(&gd.lasers[0], lasers, sizeof(lasers));
}

void DrawScreen()
{
	plotScreenRLE(0,0,SCREENDATA, COLORDATA);
}

void InitMeanies()
{
	BYTE idx;
	for (idx = 0; idx < COUNT_OF (gd.meanies); idx++)
	{
		// Refrence the meanie
		Meanie* pMeanie = &(gd.meanies[idx]);
		
		// Create the sprite
		pMeanie->spriteIndex = SSSCREATE(16,SPRITEDEF_HEIGHT_16
									| SPRITEDEF_WIDTH_16
									| SPRITEDEF_FLOATY_FLOAT
									| SPRITEDEF_FLOATX_FLOAT);		
		SSSANIM(MULTICOLOR|COLOR_WHITE, BIGDUDEBITS);

		// Set sprite bitmap
		ResetMeanie(pMeanie);
	}
}

void ResetMeanie(Meanie* pMeanie)
{
	BYTE xPos;
	pMeanie->patternIndex = COUNT_OF(gMovePattern) - 1;
	xPos = 0xFE & (PLAYFIELD_BOUNDS_LEFT + ((BYTE)(rand() % PLAYFIELD_WIDTH)));
	SSSUSE(pMeanie->spriteIndex);
	SPRITEDEF[pMeanie->spriteIndex] |= SPRITEDEF_ENABLE;
	SSSMOVEXY(xPos, 16);
	pMeanie->state = MS_ACTIVE;
}

void UpdateMeanies()
{
	BYTE idx;
	BYTE meanieIndex;
	for (idx = 0; idx < COUNT_OF (gd.meanies); idx++)
	{
		// Refrence the meanie
		Meanie* pMeanie = &(gd.meanies[idx]);

		if (SPRITEY[pMeanie->spriteIndex] > PLAYAREA_BOTTOM)
			ResetMeanie(pMeanie);

		SPRITEX[pMeanie->spriteIndex] += LNIBBLE(gMovePattern[pMeanie->patternIndex]);
		SPRITEY[pMeanie->spriteIndex] += HNIBBLE(gMovePattern[pMeanie->patternIndex]);
		SSSTOUCH(pMeanie->spriteIndex);

		// Temp code to recycle pattern
		if (pMeanie->patternIndex == 0)
			pMeanie->patternIndex = COUNT_OF(gMovePattern) - 1;
		else
			pMeanie->patternIndex--;
	}
}
 
void UpdateLasers()
{
	BYTE bulletX;
	BYTE idx;
	for (idx = 0; idx < COUNT_OF(gd.lasers); idx++)
	{
		// Refrence the meanie
		Laser* pLaser = &(gd.lasers[idx]);
		
		if ((pLaser->bulletX != LASER_IDLE) && (gd.lastKey == pLaser->kbCode))
		{
			pLaser->bulletX = 0;
			bordercolor (4);
		}

		if (pLaser->bulletX == LASER_IDLE)
			continue;
		else
			pLaser->bulletX = pLaser->bulletX + 1;
		
		if (pLaser->bulletX < 20)
		{
			bulletX = pLaser->X + (pLaser->bulletX * pLaser->direction);
			SSSCELL(bulletX, pLaser->Y, 42);
		}
			
		// Finished
		if (pLaser->bulletX > 30)
		{
			pLaser->bulletX == LASER_IDLE;
		}
		
	}
	
}

void InitScreenGeometry()
{
	// Set the screen colours
	bordercolor (COLOR_BLACK);
	bgcolor (COLOR_BLACK);
	auxColour(COLOR_BLUE);
}

void auxColour(BYTE c)
{
	VIC.volume_color = (c << 4) | (VIC.volume_color & 0x0F);
}

void plotScreenRLE(BYTE x, BYTE y, BYTE* pCharData, BYTE* pColourData)
{
	plotValuesRLE(x,y,pColourData, TRUE);
	plotValuesRLE(x,y,pCharData, FALSE);
}

void plotValuesRLE(BYTE x, BYTE y, BYTE* pData, BOOL isColour)
{
	int index = 0;
	int screenIndex = 0;
	int rleCount = 0;
	BYTE rleIndex = 0;
	BYTE value = 0;

	while (pData[index] != SCREENDATA_ENDMARKER)
	{
		if (pData[index] >=  SCREENDATA_RLEINDICATOR)
		{
			value = pData[index] & ~SCREENDATA_RLEINDICATOR;
			rleCount = 1;
		}
		else
		{
			rleCount = pData[index];
			index++;
			value = pData[index];
		}

		for (rleIndex = 0; rleIndex < rleCount; rleIndex++)
		{
			if (isColour)
			{
				PLAYCOLOR[screenIndex] = (1 << 5) | value;
			}
			else
			{
				PLAYFIELD[screenIndex] = value;
			}
			screenIndex++;
		}
		
		index++;
	}

}


