//*********************************************************************
//some pertinent VIC 20 symbols from vic-sss-mmx.h

#define RVSFLAG		0xC7		// character reverse flag
#define PLAYROWS	0xC8		// current screen row length (16-24)
#define CURSOR		0xCC		// cursor enable (0=flash)
#define CRSRCHAR	0xCE		// character under cursor
#define SCRNLINE	0xD1		// pointer to cursor's screen line
#define CRSRCOL		0xD3		// position of cursor on screen line
#define PLAYCOLS	0xD5		// current screen line length (16-24)
#define CRSRROW		0xD6		// screen row where cursor is
#define COLORLINE	0xF3		// pointer to cursor's color line
#define COLORCODE	0x0286		// current cursor color
#define CRSRCOLOR	0x0287		// color under cursor
#define SHIFTMODE	0x0291		// 0=allow, 128=locked
#define SCROLLFLAG	0x0292		// auto scrolldown flag
#define FPS			0x0285		// number of VIC re-directions every 64-jiffies
#define PENDING		0x0293		// next video page: $10 or $12
#define ACTUAL		0x0294		// save VIC startup video page
#define VSYNC		0x0295		// set when waiting for vertical sync(s)
#define VSYNC2		0x0296		// frames skipped
#define VCOUNT		0x0297		// current SSSFLIP count
#define SSSCLIPX	0x0298		// pixels to right border: 8 * (PLAYCOLS + 2)
#define SSSCLIPY	0x0299		//; pixels to bottom border: 8 * (PLAYROWS + 2)

//; matrix definition:
//; bit 0: height		0 = 8px; 1 = 16px
//; bit 1: width		0 = 8px; 1 = 16px
//; bit 2: float Y	0=fixed cell; 1=vertical float
//; bit 3: float X	0=fixed cell; 1=horizontal float
//; bit 4: repeat		0=independent; 1=re-use previous
//; bit 5: ghost		0=merge image; 1=invert image
//; bit 6: collision	0=ignore; 1=detect
//; bit 7: enabled	0=invisible; 1=visible

#define SPRITEDEF_HEIGHT_8 (0 << 0)
#define SPRITEDEF_HEIGHT_16 (1 << 0)
#define SPRITEDEF_WIDTH_8 (0 << 1)
#define SPRITEDEF_WIDTH_16 (1 << 1)
#define SPRITEDEF_FLOATY_FIXED (0 << 2)
#define SPRITEDEF_FLOATY_FLOAT (1 << 2)
#define SPRITEDEF_FLOATX_FIXED (0 << 3)
#define SPRITEDEF_FLOATX_FLOAT (1 << 3)
#define SPRITEDEF_REPEAT  (1 << 4)
#define SPRITEDEF_GHOST  (1 << 5)
#define SPRITEDEF_COLLISION  (1 << 6)
#define SPRITEDEF_ENABLE  (1 << 7)

// Make sure same as defined in VIC-SSS-MMX.h
#define SSSNULL  0x020


//API - see VIC SSS docs for more INFO
extern void  __fastcall__ IrqSetup(void);
extern void  __fastcall__ SSSINIT(void);
extern BYTE  __fastcall__ SSSCREATE(BYTE spriteH, BYTE spriteDef);
extern BYTE  __fastcall__ SSSCLEAR(BYTE characterCode);
extern void  __fastcall__ SSSPLOT(BYTE x, BYTE y);
extern void  __fastcall__ SSSPLOTS(BYTE x, BYTE y);
extern void  __fastcall__ SSSPRINT(BYTE characterCode);
extern void  __fastcall__ SSSPRINTF(const char* text);
extern void  __fastcall__ SSSPEEK(BYTE x, BYTE y);
extern void  __fastcall__ SSSPEEKS(BYTE x, BYTE y);
extern void  __fastcall__ SSSPEEKXY(BYTE x, BYTE y);
extern void  __fastcall__ SSSFLIP(BYTE frames);
extern void  __fastcall__ SSSUSE(BYTE spriteNum);
extern void  __fastcall__ SSSANIM(BYTE color, BYTE imageData[]);
extern void  __fastcall__ SSSMOVEXY(BYTE x, BYTE y);
extern void  __fastcall__ SSSTOUCH(BYTE spriteNum);
extern void  __fastcall__ SSSREFRESH(void);
extern void  __fastcall__ SSSCELL(BYTE x, BYTE y, BYTE characterCode);

// Inline helpers
void SSSPRINTTEXT(BYTE x, BYTE y, const char* text)
{
	SSSPLOT(x,y);
	SSSPRINTF(text);
}
           
